@@ -1,8 +1,8 @@
-# Instruction set for swirl.R's "virtual machine". 
+# Instruction set for swirl.R's "virtual machine".
 
 # All classes first Output, all in the same way, hence one method
 # suffices.
-# 
+#
 present <- function(current.row, e)UseMethod("present")
 
 present.default <- function(current.row, e){
@@ -10,6 +10,8 @@ present.default <- function(current.row, e){
   is_mult <- is(e$current.row, "mult_question")
   # Present output to user
   swirl_out(current.row[, "Output"], skip_after=!is_mult)
+  # Initialize attempts counter, if necessary
+  if(!exists("attempts", e)) e$attempts <- 1
   # Increment pointer
   e$iptr <- 1 + e$iptr
 }
@@ -55,9 +57,9 @@ waitUser.video <- function(current.row, e){
 }
 
 waitUser.figure <- function(current.row, e){
-  file.path <- paste(e$path,current.row[,"Figure"],sep="/")
+  fp <- file.path(e$path, current.row[,"Figure"])
   local({
-    source(file.path,local=TRUE)
+    source(fp,local=TRUE)
     xfer(environment(), globalenv())
     temp <- as.list(environment())
     e$snapshot <- c(e$snapshot, temp)
@@ -65,7 +67,7 @@ waitUser.figure <- function(current.row, e){
   readline("...")
   e$row <- 1 + e$row
   e$iptr <- 1
-} 
+}
 
 
 waitUser.mult_question <- function(current.row, e){
@@ -73,7 +75,7 @@ waitUser.mult_question <- function(current.row, e){
   # Use select.list to get the user's choice.
   choices <- strsplit(current.row[,"AnswerChoices"],";")
   # Strsplit returns a list but we want only its first element,
-  # a vector of choices. Use str_trim (pkg stringr) to remove 
+  # a vector of choices. Use str_trim (pkg stringr) to remove
   # leading and trailing white space from the choices.
   choices <- str_trim(choices[[1]])
   # Store the choice in e$val for testing
@@ -100,13 +102,58 @@ waitUser.cmd_question <- function(current.row, e){
   e$iptr <- 1 + e$iptr
 }
 
+#' @importFrom tools file_path_sans_ext
+waitUser.script <- function(current.row, e){
+  # If this is the first attempt or the user wants to start over, 
+  # then create temp files so nothing gets overwritten
+  if(e$attempts == 1 || isTRUE(e$reset)) {
+    # Get original script name
+    orig_script_name <- current.row[,"Script"]
+    # Get file path of original script
+    orig_script_path <- file.path(e$path, "scripts", orig_script_name)
+    # Path temp copy of original script
+    e$script_temp_path <- file.path(tempdir(), orig_script_name)
+    
+    # Original correct script name
+    correct_script_name <- paste0(
+      tools::file_path_sans_ext(orig_script_name), "-correct.R")
+    # Original correct script path
+    correct_script_path <- file.path(e$path, "scripts", correct_script_name)
+    # Path of temp correct script
+    e$correct_script_temp_path <- file.path(tempdir(), correct_script_name)
+    
+    # Copy original script to temp file
+    file.copy(orig_script_path, e$script_temp_path, overwrite = TRUE)
+    # Copy original correct to temp correct
+    file.copy(correct_script_path, e$correct_script_temp_path, overwrite = TRUE)
+    
+    # Set reset flag back to FALSE
+    e$reset <- FALSE
+  }
+  # Have user edit the copy. This will reopen the file if 
+  # accidentally closed
+  file.edit(e$script_temp_path)
+  # Give instructions
+  # swirl_out("INSTRUCTIONS: Edit the script and experiment in the console as much as you want. When you are ready to move on, SAVE YOUR SCRIPT and type submit() at the prompt. The script will remain open until you close it.",
+  #          skip_before = FALSE, skip_after = TRUE)
+  # Indicate a return to the prompt is necessary
+  e$prompt <- TRUE
+  # Enter 'play' mode so that user can mess around in the console
+  e$playing <- TRUE
+  # Advance lesson
+  e$iptr <- 1 + e$iptr
+}
+
 # Only the question classes enter a testing loop. Testing is the
 # same in both cases. If the response is correct they indicate
 # instruction should progress. If incorrect, they publish a hint
-# and return to the previous step. 
+# and return to the previous step.
 testResponse <- function(current.row, e)UseMethod("testResponse")
 
 testResponse.default <- function(current.row, e){
+  # Increment attempts counter
+  e$attempts <- 1 + e$attempts
+  # Get answer tests
   tests <- current.row[,"AnswerTests"]
   if(is.na(tests) || tests == ""){
     results <- is(e, "dev")
@@ -122,6 +169,8 @@ testResponse.default <- function(current.row, e){
     swirl_out(praise())
     e$iptr <- 1
     e$row <- 1 + e$row
+    # Reset attempts counter, since correct
+    e$attempts <- 1
   } else {
     # Restore the previous global environment from the official
     # in case the user has garbled it, e.g., has typed x <- 3*x
@@ -137,17 +186,22 @@ testResponse.default <- function(current.row, e){
     temp <- current.row[,"Hint"]
     # Suppress extra space if multiple choice
     is_mult <- is(e$current.row, "mult_question")
-    if (!is.na(temp)) swirl_out(current.row[,"Hint"], skip_after=!is_mult)
+    # If hint is specified, print it. Otherwise, just skip a line.
+    if (!is.na(temp)) {
+      swirl_out(current.row[,"Hint"], skip_after=!is_mult)
+    } else {
+      message()
+    }
     e$iptr <- e$iptr - 1
   }
 }
 
 testMe <- function(keyphrase, e){
   # patch to accommodate old-style tests
-  oldcourse <- attr(e$les, "course_name") %in% 
+  oldcourse <- attr(e$les, "course_name") %in%
     c("Data Analysis", "Mathematical Biostatistics Boot Camp",
       "Open Intro")
-  
+
   if(oldcourse){
     # Use old test syntax
     # Add a new class attribute to the keyphrase using
@@ -162,14 +216,14 @@ testMe <- function(keyphrase, e){
 }
 
 # CUSTOM TEST SUPPORT. An environment for custom tests is inserted
-# "between" function testMe and the swirl namespace. That is,  
+# "between" function testMe and the swirl namespace. That is,
 # an environment, customTests, is created with parent swirl
 # and child testMe. Code evaluated within testMe will thus search
 # for functions first in customTests, and then in the swirl namespace.
 #
 # Custom tests must be defined in a file named "customTests.R" in the
-# lesson directory. Tests in such files are loaded into environment 
-# customTests when a lesson is first loaded or progress is restored. 
+# lesson directory. Tests in such files are loaded into environment
+# customTests when a lesson is first loaded or progress is restored.
 # The environment is cleared between lessons.
 
 # An environment with parent swirl to hold custom tests.
